# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-05 02:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0002_auto_20170327_1709'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='title',
            field=models.CharField(max_length=50),
        ),
    ]



from django import forms

from users.models import MemberUser


class MemberUpdate(forms.Form):
    first_name = forms.CharField(max_length=30, required=False)
    last_name = forms.CharField(max_length=30, required=False)
    email = forms.EmailField(max_length=50, widget=forms.EmailInput, required=False)
    password = forms.CharField(max_length=30, min_length=8, widget=forms.PasswordInput, required=False, initial=' ')
    phone = forms.CharField(max_length=15, show_hidden_initial='+651234578', required=False)


class MemberCreate(forms.ModelForm):
    username = forms.CharField(max_length=8, min_length=8, required=True,
                               help_text='Username should contain exactly 8 characters. (Murdoch ID)')

    class Meta:
        model = MemberUser
        fields = ['username', 'first_name', 'last_name', 'role', 'email', 'phone']



from users.models import User
import django_filters


class SearchFilter(django_filters.FilterSet):
    username = django_filters.CharFilter(lookup_expr='icontains')
    first_name = django_filters.CharFilter(lookup_expr='icontains')
    last_name = django_filters.CharFilter(lookup_expr='icontains')

    class Meta:
        model = User
        fields = ['username', 'first_name', 'last_name']



from django.shortcuts import render, HttpResponse

from users.models import Project, AdminUser, LeaderUser

from .parse_to_html import parse_log
from yattag import Doc


# Create your views here.
def test(request, username, project):
    user = ''
    if AdminUser.objects.filter(username__exact=username).count() is 1:
        user = AdminUser.objects.get(username__exact=username)
    else:
        user = LeaderUser.objects.get(username__exact=username)

    if Project.objects.filter(name__exact=project).exists():
        proj = Project.objects.get(name__exact=project)
    elif Project.objects.filter(leader__username=username):
        proj = user.project
    else:
        print(username)
        print('------->>' + project + '<<---------')
        return HttpResponse("No project created yet. Please create a project first.")

    doc, tag, text = Doc().tagtext()
    with tag('h3', id='main-title'):
        text(project)

    p = parse_log(proj)
    return render(request, 'log/test.html',
                  {'log_data': p.test(),
                   'project_title': doc.getvalue(),
                   'title': proj.name,
                   'section': 'Log',
                   'user': user
                   })



"""
Django settings for HexOmega project.

Generated by 'django-admin startproject' using Django 1.10.5.

For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

import os
import pickle
import time

import schedule

import users

# from users.utils import start_schedule_thread

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'f=zxeatkxp9*a18v(mr8wt+6-2b2)s643enssg4tp(q%_1&(g2'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = [
    'localhost',
    '127.0.0.1',
    '*'
]

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'guardian',
    'whitenoise',
    'bootstrap3',
    'annoying',
    'users',
    'log',
    
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'whitenoise.middleware.WhiteNoiseMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'HexOmega.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')]
        ,
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'HexOmega.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.10/ref/settings/#databases

if False is True:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
        }
    }
else:
    import dj_database_url
    DATABASES = dict()
    DATABASES['default'] = dj_database_url.config()

# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'Asia/Singapore'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static')

# auth backend.
AUTHENTICATION_BACKENDS = [
    # 'users.backends.CustomUserAuth',
    'django.contrib.auth.backends.ModelBackend',  # this is default
    'guardian.backends.ObjectPermissionBackend'
]

AUTH_USER_MODEL = 'users.User'

GUARDIAN_MONKEY_PATCH = False

SESSION_EXPIRE_AT_BROWSER_CLOSE = True

LOGIN_REDIRECT_URL = '/login/'
LOGIN_URL = '/login/'

# Mail settings
o = open('HexOmega/pwd.dump', 'rb')
EMAIL_HOST = 'smtp.yandex.com'
EMAIL_PORT = 465
EMAIL_USE_SSL = True
EMAIL_HOST_USER = 'hex.omega@yandex.com'
EMAIL_HOST_PASSWORD = pickle.load(o)

SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')



from ajax_select.fields import AutoCompleteSelectMultipleField
from django import forms
from django.forms import ModelForm
from users.models import AdminUser, Project


class CreateProjectForm(ModelForm):
    class Meta:
        model = Project
        fields = ['name', 'start_date', 'end_date', 'description', 'admins']
        widgets = {
            'start_date': forms.DateTimeInput(attrs={'class': 'datetime-input'}),
            'end_date': forms.DateTimeInput(attrs={'class': 'datetime-input'}),
        }
        help_texts = {
            'start_date': "Please use this format: 'YYYY-MM-DD'",
            'end_date': "Please use this format: 'YYYY-MM-DD'",
            'admins': 'Hold Control to select multiple choices'
        }



from django.test import TestCase
from users.models import AdminUser, LeaderUser, MemberUser, User, Project, Role
from users.Claudia.user_form import AdminUserForm, AdminUpdateForm, UserUpdateForm


class AdminModelTest(TestCase):
    def setUp(self):
        adm01 = AdminUser.objects.create_user(username='12341234', first_name='Aaron', last_name='Hotchner', email='aaron_hotchner@gmail.com', phone='88770000')
        adm01.save()
        adm02 = AdminUser.objects.create_user(username='22334455', first_name='Megan', last_name='Smith', email='megan_smith@gmail.com', phone='98701122')
        adm02.save()

    def test_phone_max_length(self):
        user = AdminUser.objects.get(pk=2)
        max_length = user._meta.get_field('phone').max_length
        self.assertEquals(max_length, 15)


class AdminUserFormTest(TestCase):
    def test_invalid_adminuser_form(self):
        username = "12345678900"
        first_name = "Megan"
        last_name = "Smith"
        email = "megan_smith@gmail.com"
        password = "qwerty123"
        phone = "98790001"
        bio = "Test"
        form_data = {'username': username, 'first_name': first_name, 'last_name': last_name, 'email': email,
                     'password': password, 'phone': phone, 'bio': bio}
        form = AdminUserForm(data=form_data)
        self.assertFalse(form.is_valid())

    def test_valid_adminuser_form(self):
        username = "12345678"
        first_name = "Megan"
        last_name = "Smith"
        email = "megan_smith@gmail.com"
        password = "qwerty123"
        phone = "98790001"
        bio = "Test"
        form_data = {'username': username, 'first_name': first_name, 'last_name': last_name, 'email': email,
                     'password': password, 'phone': phone, 'bio': bio}
        form = AdminUserForm(data=form_data)
        self.assertTrue(form.is_valid())


class AdminUpdateFormTest(TestCase):
    def setUp(self):
        adm01 = AdminUser.objects.create_user(username='12341234', first_name='Aaron', last_name='Hotchner', email='aaron_hotchner@gmail.com', phone='88770000')
        adm01.save()
        adm02 = AdminUser.objects.create_user(username='22334455', first_name='Megan', last_name='Smith', email='megan_smith@gmail.com', phone='98701122')
        adm02.save()

    def test_invalid_adminupdate_form(self):
        user = AdminUser.objects.get(pk=2)
        num = '202-555-0137-019890'
        form_data = {'first_name': user.first_name, 'last_name': user.last_name, 'email': user.email, 'phone': num}
        form = AdminUpdateForm(data=form_data)
        self.assertFalse(form.is_valid())

    def test_valid_adminupdate_form(self):
        user = AdminUser.objects.get(pk=2)
        email = 'derek_morgan@gmail.com'
        num = '8555-0137'
        form_data = {'first_name': user.first_name, 'last_name': user.last_name, 'email': email, 'phone': num}
        form = AdminUpdateForm(data=form_data)
        self.assertTrue(form.is_valid())


class UserUpdateFormTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()
        lead = LeaderUser.objects.create_user(username='12341234', first_name='Aaron', last_name='Hotchner', email='aaron_hotchner@gmail.com', phone='88770000')
        lead.save()

    def test_invalid_userupdate_form(self):
        user = User.objects.get(first_name__iexact='Derek')
        num = '202-555-0137-019890'
        form_data = {'first_name': user.first_name, 'last_name': user.last_name, 'email': user.email, 'phone': num}
        form = UserUpdateForm(data=form_data)
        self.assertFalse(form.is_valid())

    def test_valid_userupdate_form(self):
        user = User.objects.get(first_name__iexact='Aaron')
        num = '202-555-0137'
        email = 'aaron_hotchner@example.com'
        form_data = {'first_name': user.first_name, 'last_name': user.last_name, 'email': email, 'phone': num}
        form = UserUpdateForm(data=form_data)
        self.assertTrue(form.is_valid())


class CreateAdminUserViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()

    def test_create_admin_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/add_admin/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'users/adminuser_form.html')


class GetAdminDetailViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()

    def test_get_admin_detail_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/details/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'users/user_information.html')


class UpdateAdminDetailViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()

    def test_update_admin_detail_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/update/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'users/update_admin_form.html')


class DisplayAllProjectsViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()

    def test_display_all_projects_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/project/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'users/all_project_list.html')


class DisplayOpenProjectsViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()

    def test_display_open_projects_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/open_project/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'users/open_project_list.html')


class DisplayProjectInformationViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()
        lead = LeaderUser.objects.create_user(username='12341234', first_name='Aaron', last_name='Hotchner',
                                              password='qwerty123', email='aaron_hotchner@gmail.com', phone='88770000')
        lead.save()
        project = Project.objects.create(name='PMT', start_date='2017-03-01', end_date='2017-03-30', leader_id=lead.id)
        project.admins.add(adm)
        project.save()

    def test_display_project_information_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/PMT/project_detail/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'users/project_information.html')


class UserUpdateViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()
        lead = LeaderUser.objects.create_user(username='12341234', first_name='Aaron', last_name='Hotchner',
                                              password='qwerty123', email='aaron_hotchner@gmail.com', phone='88770000')
        lead.save()
        project = Project.objects.create(name='PMT', start_date='2017-03-01', end_date='2017-03-30', leader_id=lead.id)
        project.admins.add(adm)
        project.save()
        role = Role.objects.create(title='Security Coordinator')
        role.save()
        mem = MemberUser.objects.create_user(username='67091200', first_name='Spencer', last_name='Reid',
                                             password='qwerty123', email='spencer_reid@gmail.com', phone='88779876',
                                             role_id=role.id, project_id=project.id)
        mem.save()

    def test_user_update_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/update_user/12341234/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'users/user_update_form.html')


class DeleteAUserViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()
        lead = LeaderUser.objects.create_user(username='12341234', first_name='Aaron', last_name='Hotchner',
                                              password='qwerty123', email='aaron_hotchner@gmail.com', phone='88770000')
        lead.save()

    def test_delete_a_user_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/delete/12341234/')
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, '/admin_user/G8009720/list/')


class DisplayListOfUsersViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()

    def test_display_list_of_users_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/list/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'users/list_of_users.html')


class GetUserDetailViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()
        lead = LeaderUser.objects.create_user(username='12341234', first_name='Aaron', last_name='Hotchner',
                                              password='qwerty123', email='aaron_hotchner@gmail.com', phone='88770000')
        lead.save()

    def test_get_user_detail_view_test(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/user_detail/12341234/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'users/user_information.html')


class SearchUsersViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()

    def test_search_users_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/search/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'users/search.html')


class DeleteProjectViewTest(TestCase):
    def setUp(self):
        adm = AdminUser.objects.create_user(username='G8009720', first_name='Derek', last_name='Morgan',
                                            password='qwerty123', email='derek_morgan@gmail.com', phone='97706000')
        adm.save()
        lead = LeaderUser.objects.create_user(username='12341234', first_name='Aaron', last_name='Hotchner',
                                              password='qwerty123', email='aaron_hotchner@gmail.com', phone='88770000')
        lead.save()
        project = Project.objects.create(name='PMT', start_date='2017-03-01', end_date='2017-03-30', leader_id=lead.id)
        project.admins.add(adm)
        project.save()

    def test_delete_project_view(self):
        user_login = self.client.login(username='G8009720', password='qwerty123')
        self.assertTrue(user_login)
        response = self.client.get('/admin_user/G8009720/delete_project/PMT/')
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, '/admin_user/G8009720/project/')


import mimetypes

from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.mail import send_mail
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.shortcuts import render, redirect
from django.http import HttpResponse
from django.contrib.auth import login, logout, update_session_auth_hash
from django.contrib.auth.decorators import login_required
from django.views.generic.edit import UpdateView, CreateView
from django.views.generic.detail import DetailView
from django.views.generic.list import ListView
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.contrib.auth.models import Group

from HexOmega.settings import BASE_DIR
from .utils import get_default_password, mail_kickoff, uploaded_file_handler
from .models import Project, AdminUser, MemberUser, LeaderUser, Task, User
from .backends import CustomUserAuth
from .forms.login_form import LoginForm
from .forms.project_forms import CreateProjectForm
from .forms.task_forms import CreateTaskForm, LeaderUpdateTaskForm
from .forms.member_form import MemberUpdate, MemberCreate

from .Xav.user_context import url_context

from log.Log import log

import os
from datetime import datetime, timedelta


def index(request):
    return render(request,
                  'users/index.html')


def login_auth_2(request):
    """
    Login page authentication using django forms.
    If easier and simpler, implement this else the
    stuff I threw together up there.
    :param request:
    :return:
    """
    if request.user.is_authenticated():
        return redirect('user_logged_in', request.user.username)

    if request.method == 'POST':
        form = LoginForm(request.POST)
        if form.is_valid():
            username = request.POST.get('username')
            password = request.POST.get('password')
            rem = request.POST.get('rem')
            user = CustomUserAuth().authenticate(username=username, password=password)

            if user is False:
                form.errors['password'] = 'The username or password is incorrect.'
                return render(request,
                              'users/login.html',
                              {
                                  'form': form,
                                  'errors': form.errors
                              })

            if user is not None:
                print('User [{}] is logging in.'.format(user.username))
                login(request, user, backend='django.contrib.auth.backends.ModelBackend')
                if rem is not None:
                    request.session.set_expiry(7200)
                else:
                    request.session.set_expiry(1800)
                return redirect('user_logged_in', username=username)

    else:
        form = LoginForm()

    return render(request, 'users/login.html', {'form': form})


@login_required
def logged_in(request, username):
    if AdminUser.objects.filter(username__exact=username).count() == 1:
        return redirect('open_project', username)
    elif LeaderUser.objects.filter(username__exact=username).count() == 1:
        return redirect('leader_home', username)
    else:
        user = MemberUser.objects.get(username__exact=username)
        return redirect('task_list', username)


@login_required
def jump_ship(request):
    print('jumping ship....')
    logout(request)
    return redirect('login_page')


@login_required
def delete_user(request, username, d):
    if User.objects.get(username__iexact=d):
        person = User.objects.get(username__iexact=d)
        from django.db import IntegrityError
        try:
            if d != '10101010':
                person.delete()
        except IntegrityError as ie:
            from django.contrib import messages
            messages.add_message(request, messages.WARNING,
                                 'Cannot delete leader without closing and deleting a project.')
    return redirect('list_of_users', username)


@login_required
def member_upload(request, username, task):
    t = Task.objects.get(title=task)
    done = None
    if 'up_file' in request.FILES:
        if t.est_end.date() > datetime.now().date():
            t.deliverable = request.FILES['up_file']
            t.save()
            mail_kickoff(MemberUser.objects.get(username__exact=username), t, var=3)
            log('SUCCESS', MemberUser.objects.get(username__exact=username),
                '{} uploaded a deliverable for {}'.format(username, t.title))
            print(t.deliverable.url)
            return redirect('task_list', username)
        else:
            done = True
            return render(request, 'list.html', {
                'username': username,
                'done': True
            })
    else:
        print('No file!!')
    return render(request, 'list.html', {
        'username': username,
        'done': done
    })


@login_required
def list_users(request, username):
    return render(request, 'list.html', {'admins': AdminUser.objects.all()})


@login_required
@url_context
def get_list_of_users(request):
    """
    Display a list of admin users
    /list/
    :param request:
    :return:
    :author Caroline
    """
    admin_user_list = AdminUser.objects.order_by('pk')
    paginator = Paginator(admin_user_list, 1)  # Show 3 admin per page

    page = request.GET.get('page')
    try:
        admin_list = paginator.page(page)
    except PageNotAnInteger:
        # If page is not an integer, deliver first page.
        admin_list = paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        admin_list = paginator.page(paginator.num_pages)
    context = {'admin_list': admin_list, 'page': page}
    return render(request, 'users/list_of_users.html', context)


# ============================================================================
# Release Me!
@login_required
def leader_home(request, username):
    user = LeaderUser.objects.get(username__exact=username)
    try:
        tasks = user.project.actionlist.task_set.all()
        for task in Task.objects.filter(action_list__project__leader__username=username):
            print(task.title, task.action_list.project.name)
            # print(task.deliverable.url)
    except Exception as e:
        print('Ahhhhhh')
        tasks = None
    return render(request, 'leader_home.html', {'user': user, 'tasks': tasks})


class CreateMember(CreateView, LoginRequiredMixin):
    # fields = ['username', 'first_name', 'last_name', 'role', 'email', 'phone']
    form_class = MemberCreate
    username = ''
    model = MemberUser
    l = None
    template_name = 'create_member.html'

    def form_valid(self, form):
        try:
            form.instance.project = self.l.project
        except self.l.project.DoesNotExist as np:
            # from django.contrib import messages
            messages.add_message(self.request, messages.WARNING,
                                 'Cannot add members to a project without a project.\nPlease create a project first.')
        password = get_default_password()
        form.instance.set_password(password)
        mail_kickoff(form.instance, password)
        messages.add_message(self.request, messages.INFO, 'User [{}] created.'.format(form.instance.username))
        update_session_auth_hash(self.request, self.request.user)
        return super(CreateMember, self).form_valid(form)

    def get_form_kwargs(self):
        self.l = LeaderUser.objects.get(username__exact=self.request.user.username)
        p = self.request.get_full_path()
        print(p)
        self.success_url = '/'.join(p.split('/')[:-1]) + '/'
        kwargs = super(CreateMember, self).get_form_kwargs()
        # kwargs['pn'] = l.project.name
        return kwargs


class MemberHome(DetailView, LoginRequiredMixin):
    model = MemberUser
    username = ''
    template_name = 'member_home.html'

    def get_object(self, queryset=None):
        return MemberUser.objects.get(username=self.kwargs.get('username'))

    def get_context_data(self, **kwargs):
        context = super(MemberHome, self).get_context_data(**kwargs)
        return context


@login_required
def show_tasks(request, username):
    ts = Task.objects.filter(members__username=username)
    print(ts)
    return render(request, 'list.html', {'tasks': ts})


# ============================================================================
# My project and tasks modules
# 2017-03-22

def get_project_path(p):
    return os.path.join(BASE_DIR,
                        os.path.join('projects', p.name + '/'))


@login_required
def create_project(request, username):
    if request.method == 'POST':
        form = CreateProjectForm(request.POST)
        if form.is_valid():
            p = form.save(commit=False)
            p.leader = LeaderUser.objects.get(username__exact=username)
            p.save()
            for a in request.POST.get('admins'):
                p.admins.add(a)
            path = get_project_path(p)
            # os.makedirs(path, 0o755)
            if not os.path.exists(path):
                os.makedirs(path, 0o755)
            if not os.path.exists(os.path.join(path, 'activity.log')):
                f = open(os.path.join(path, 'activity.log'), 'w+')
                f.close()
        return redirect('display_leader', username)
    else:
        form = CreateProjectForm()

    return render(request, 'crproj.html', {'form': form})


@login_required
def create_task(request, username):
    l = LeaderUser.objects.get(username__exact=username)
    if request.method == 'POST':
        form = CreateTaskForm(request.POST)
        if form.is_valid():
            mem_dat = form.cleaned_data.get('members')
            title = form.cleaned_data.get('title')
            est_end = form.cleaned_data.get('est_end')
            status = form.cleaned_data.get('status')
            lt = form.cleaned_data.get('to_leader')
            if lt is None:
                lt = False
            t = Task.objects.create(title=title, est_end=est_end, status=status, to_leader=lt,
                                    action_list=l.project.actionlist)
            t.save()

            for m in mem_dat:
                t.members.add(m)
            t.save()
            log('INFO', l, '{} added a new Task [{}]'.format(l.username, t.title))
            return redirect('leader_home', username)
        else:
            print(form.errors)
    else:
        form = CreateTaskForm({'pn': l.project.name})

    return render(request, 'crtask.html', {'form': form})


class TaskUpdate(UpdateView, LoginRequiredMixin):
    username = ''
    model = Task
    template_name = 'crtask.html'
    content_type = 'multipart-form-data'
    form_class = LeaderUpdateTaskForm

    def get_form_kwargs(self):
        l = LeaderUser.objects.get(username__exact=self.request.user.username)
        t = Task.objects.get(pk=self.kwargs['pk'])
        up_flag = False
        up_name = ''
        if bool(t.deliverable):
            up_flag = True
            up_name = t.deliverable.name.split('/')[-1]
            t.status = 'Completed'
            t.save()
            log('SUCCESS', l, '{} uploaded a deliverable to Task [{}]'.format(l.username, t.title))
            mail_kickoff(l, t, var=3)
        p = self.request.get_full_path()
        self.success_url = '/'.join(p.split('/')[:-3]) + '/'
        kwargs = super(TaskUpdate, self).get_form_kwargs()
        kwargs['pn'] = l.project.name
        kwargs['up_flag'] = up_flag
        kwargs['up_name'] = up_name
        log('INFO', l, '{} made changes to Task [{}]'.format(l.username, t.title))
        return kwargs


@login_required
def update_member(request, username):
    mem = MemberUser.objects.get(username__exact=username)
    form = MemberUpdate(request.POST, initial={
        'first_name': mem.first_name,
        'last_name': mem.last_name,
        'email': mem.email,
        'phone': mem.phone
    })
    if request.method == 'POST':
        if form.is_valid():
            fn = request.POST.get('first_name')
            ln = request.POST.get('last_name')
            email = request.POST.get('email')
            p = request.POST.get('password')
            ph = request.POST.get('phone')
            if fn is not '':
                mem.first_name = fn
            if ln is not '':
                mem.last_name = ln
            if email is not '':
                mem.email = email
            if (p is not None and p is not '') and len(p.strip()) >= 8:
                mem.set_password(p)
            if ph is not '':
                mem.phone = ph
            if mem.has_usable_password():
                update_session_auth_hash(request, mem)
                mem.save()
                logout(request)
                from django.contrib import messages
                messages.add_message(request, messages.INFO, 'Your profile was altered. Please login again.')
                return redirect('login_page')
        else:
            print(form.errors)
    else:
        form = MemberUpdate()

    return render(request, 'update_member.html', {
        'form': form,
        'user': mem,
        'title': 'Update'
    })


def get_list_of_members(request, username):
    member_user_list = MemberUser.objects.order_by('pk').filter(project__leader__username=username)
    user = LeaderUser.objects.get(username__iexact=username)
    paginator = Paginator(member_user_list, 10)  # Show 3 admin per page

    page = request.GET.get('page')
    try:
        member_list = paginator.page(page)
    except PageNotAnInteger:
        # If page is not an integer, deliver first page.
        member_list = paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        member_list = paginator.page(paginator.num_pages)
    context = {'member_list': member_list, 'page': page, 'user': user}
    return render(request, 'users/list_of_members.html', context)


def delete_a_member(request, username, d):
    if MemberUser.objects.get(username__iexact=d):
        person = MemberUser.objects.get(username__iexact=d)
        person.delete()
    return redirect('members_list', username)


@login_required
def project_information(request, username, p):
    print('Yoohoo!')
    project = Project.objects.get(name__exact=p)
    for p in project.actionlist.task_set.all():
        print(p.deliverable.name.split('/')[-1])
    return render(request, 'users/project_information.html', {'project': project})


@login_required
def send_file(request, username, p, task_pk):
    task = Task.objects.get(pk=int(task_pk))
    file_path = '/' + task.deliverable.url
    print(task)
    if '%20' in file_path:
        file_path = file_path.replace('%20', ' ')
    file_mimetype = mimetypes.guess_type(file_path)
    # if os.path.exists(file_path):
    try:
        with open(file_path, 'rb') as fh:
            response = HttpResponse(fh.read(), content_type=file_mimetype)
            response['Content-Disposition'] = 'attachment; filename=' + os.path.basename(file_path)
            return response
    # else:
    except Exception as e:
        return HttpResponse('File retrieval error. <br>' + file_path + '<br>' + str(e))



from django.db import models
from django.contrib.auth.models import AbstractUser
from django.contrib.auth.models import Group
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.contrib.auth.models import PermissionsMixin
import itertools
from guardian.mixins import GuardianUserMixin

import os
from HexOmega.settings import BASE_DIR


class User(AbstractUser, GuardianUserMixin):
    # username = models.CharField(max_length=30, primary_key=True)
    bio = models.TextField(max_length=500, blank=True)
    is_admin = False
    is_leader = False
    is_member = False
    phone = models.TextField(max_length=15, blank=True)
    email = models.EmailField(unique=True, blank=True)

    def __str__(self):
        return self.get_full_name()


class AdminUser(User, PermissionsMixin):
    """
    Class that hold data for every admin level user.
    Most of the fields are inherited from AbstractUser.
    """
    is_admin = True

    class Meta:
        db_table = 'AdminUser'
        permissions = (
            ('can_create_admin', 'Can create Admin'),
        )


@receiver(post_save, sender=AdminUser)
def add_admin_to_group(sender, **kwargs):
    """
    Runs as soon as any AdminUser object is saved.
    Creates/adds to the admin group and permissions are given
    to the admin group.

    :param sender: AdminUser
    :param kwargs: list of essential arguments
    :return: None
    """

    admin_group, created = Group.objects.get_or_create(name='admin_group')
    if created:
        pass
    if kwargs['instance'].username != 'AnonymousUser':
        admin_group.user_set.add(kwargs.get('instance'))


class LeaderUser(User, PermissionsMixin):
    is_leader = True

    class Meta:
        db_table = 'Leader'
        permissions = (
            ('can_create_leader', 'Can create Team Leader'),
        )


@receiver(post_save, sender=LeaderUser)
def add_leader_to_group(sender, **kwargs):
    """
    Runs as soon as any LeaderUser object is saved.
    Creates/adds to the leader group and permissions are given
    to the leader group.

    :param sender: LeaderUser
    :param kwargs: list of essential arguments
    :return: None
    """
    leader_group, created = Group.objects.get_or_create(name='leader_group')
    if created:
        pass
    if kwargs['instance'].username != 'AnonymousUser':
        leader_group.user_set.add(kwargs.get('instance'))


class Role(models.Model):
    title = models.CharField(max_length=30, unique=True)

    def __str__(self):
        return self.title

    class Meta:
        db_table = 'Role'


class Project(models.Model):
    """
    Model changelog:
        1.  have to add null=True for the date fields.
    """

    @property
    def get_name(self):
        return self.name

    name = models.CharField(max_length=50, blank=False, unique=True)
    STATUS_CHOICES = (
        ('Open', 'Open'),
        ('Closed', 'Closed')
    )
    status = models.CharField(choices=STATUS_CHOICES, default=0, max_length=7)
    start_date = models.DateField(blank=True)
    end_date = models.DateField(blank=True)
    description = models.TextField(max_length=500, blank=True)
    leader = models.OneToOneField(LeaderUser, models.DO_NOTHING)
    admins = models.ManyToManyField(AdminUser)

    def path(self):
        return os.path.join(BASE_DIR,
                            os.path.join('projects', self.name + '/'))

    def __str__(self):
        return self.name

    class Meta:
        db_table = 'Project'


@receiver(post_save, sender=Project)
def add_activitylog(sender, instance, created, **kwargs):
    if created:
        print(instance.name)
        c = os.path.join(BASE_DIR,
                         os.path.join('projects',
                                      os.path.join(instance.name, 'activity.log')))
        print(c)
        print('/'.join(c.split('/')[:-1]))
        instance.path = '/'.join(c.split('/')[:-1]) + '/'
        # instance.save()
        ActivityLog.objects.create(title=instance.name, project=instance, content=c)
        if not os.path.exists('/'.join(c.split('/')[:-1])):
            os.makedirs('/'.join(c.split('/')[:-1]), 0o755)
        f = open(c, 'w+')
        f.close()


@receiver(post_save, sender=Project)
def add_actionlist(sender, instance, created, **kwargs):
    if created:
        ActionList.objects.create(name=instance.name, project=instance)


class ActionList(models.Model):
    name = models.CharField(max_length=30)
    project = models.OneToOneField(Project)
    description = models.TextField(max_length=500, blank=True)

    def __str__(self):
        return self.name

    class Meta:
        db_table = 'ActionList'


class MemberUser(User, PermissionsMixin):
    role = models.ForeignKey(Role, models.DO_NOTHING)
    project = models.ForeignKey(Project, models.DO_NOTHING)
    # tasks = models.ManyToManyField(Task)
    is_member = True

    class Meta:
        db_table = 'Member'
        permissions = (
            ('can_create_member', 'Can create Team Member'),
        )


def get_path(instance, filename):
    p = instance.action_list.project.path()
    print(p)
    if type(filename) is str:
        f = os.path.join(p, filename)
    else:
        f = os.path.join(p, filename.name)
    print(f)
    return f


def get_student_list(project):
    m = project.memberuser_set.all()
    l = project.leader
    students = list(m)
    students.append(l)
    for s in students:
        print(str(s.username) + ' -- ' + str(type(s)))
    return [s.username for s in students]


class Task(models.Model):
    """
        Model Changelog:
            1.  have to add members_assigned as a many to many relation.
                (a task can have many members and many tasks can be done by
                one member)
    """
    TASK_STATUS_CHOICES = (
        ('Draft', 'Draft'),
        ('Final', 'Final'),
        ('Unassigned', 'Unassigned'),
        ('Assigned', 'Assigned'),
        ('Completed', 'Completed'),
        ('Late', 'Late'),
        ('Not Submitted', 'Not Submitted')
    )
    status = models.CharField(choices=TASK_STATUS_CHOICES, max_length=14, default=2, blank=False)
    start_date = models.DateTimeField(auto_now=True)
    est_end = models.DateTimeField(null=True, blank=True)
    actual_end = models.DateTimeField(null=True, blank=True)
    description = models.TextField(max_length=500, null=True, blank=True)
    link = models.URLField(null=True, blank=True)
    title = models.CharField(max_length=50)
    action_list = models.ForeignKey(ActionList, models.CASCADE, null=True, blank=True)
    deliverable = models.FileField(upload_to=get_path, blank=True, null=True)
    to_leader = models.NullBooleanField(verbose_name="Is the leader working on this Task?", null=True)
    members = models.ManyToManyField(MemberUser)

    def __str__(self):
        return self.title

    class Meta:
        db_table = 'Task'


class ActivityLog(models.Model):
    title = models.CharField(max_length=30)
    # the content variable is the path to the actual log file.
    content = models.TextField(max_length=100, blank=True, null=True)
    project = models.OneToOneField(Project)
    timestamp = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.title

    class Meta:
        db_table = 'ActivityLog'

# @receiver(post_save, sender=MemberUser)
# def add_member_to_group(sender, instance, created, **kwargs):
#     """
#     Runs as soon as any MemberUser object is saved.
#     Creates/adds to the member group and permissions are given
#     to the member group.
#
#     :param created:
#     :param instance: instance of member user
#     :param sender: MemberUser
#     :param kwargs: list of essential arguments
#     :return: None
#     """
#     if created and instance.username != 'AnonymousUser':
#         # member_group.user_set.add(kwargs.get('instance'))
#         instance.groups.add(Group.objects.get_or_create(name='member_group'))



from django.conf.urls import url, include

from .views import *

urlpatterns = [
    url(r'^leader_user/(?P<username>[A-Z0-9][0-9]{7})/(?P<project>[A-Za-z]+)/log/$', test, name='leader_log'),
    url(r'^admin_user/(?P<username>[A-Z0-9][0-9]{7})/(?P<project>[A-Za-z]+)/log/$', test, name='admin_log'),
]



from datetime import datetime

from users.models import MemberUser, LeaderUser, AdminUser


# To do:
#   1.  test the log function, once others are done with the module.
#   2.  add a function to parse each line of a given log file and
#       render them into a HTML structure for use in a view.

def log(level, user, content, **kwargs):
    """
    Log function.
    Call from any function or context with the required parameters to
    write to the appropriate log file. Can use threading for pseudo
    non-blocking IO but due to some constraints have reasoned otherwise.

    :param level: either: INFO, WARNING or SUCCESS
    :param user: user who called a function with log.
    :param content: the path to the log
    :param kwargs: random kwargs to add to the log message.
    Will add implementation only if needed.
    :return: None
    """
    # the content attribute of the project has the entire
    # path from BASE_DIR(mentioned in settings.py).
    # l = MemberUser.objects.get(username__contains=username)
    # print(l.username)
    # INFO = 'INFO'
    # WARNING = 'WARNING'
    # SUCCESS = 'SUCCESS'
    access_level = None
    if user.is_admin:
        access_level = 'ADMIN'
    elif user.is_leader:
        access_level = 'LEADER'
    else:
        access_level = 'MEMBER'

    data = datetime.now().strftime('%A, %d. %B %Y %I:%M%p') + ' '
    # data += content

    logfile = user.project.activitylog.content

    if logfile is '':
        raise ValueError('Empty path to log file.')
    else:
        f = open(logfile, 'a')
        print('[{}] [{}] [{}] [{}] [{}] [{}]'.format(level, user.get_full_name(), access_level, user.project.name, data,
                                                     content),
              file=f)
        f.flush()
        f.close()



from users.views import *

from .add_leader_form import *

from django.db.utils import IntegrityError


def create_leader_user(request, username):
    form = LeaderForm()
    if request.method == 'POST':
        form = LeaderForm(request.POST)
        if form.is_valid():
            username = request.POST.get('username')
            first_name = request.POST.get('first_name')
            last_name = request.POST.get('last_name')
            email = request.POST.get('email')
            password = get_default_password()
            try:
                user = LeaderUser.objects.create_user(username=username, first_name=first_name, last_name=last_name,
                                                      email=email, password=password)
            except IntegrityError as e:
                return render(request, 'users/leaderuser_form.html',
                              {'form': form, 'mail_error': 'The email is not unique!'})

            user.set_password(password)
            mail_kickoff(user, password)
            user.save()
            update_session_auth_hash(request, request.user)
            return redirect('display_admin', request.user.username)
    return render(request, 'users/leaderuser_form.html', {'form': form})


@login_required
def display_leader_detail(request, username):
    user = LeaderUser.objects.get(username__iexact=username)
    return render(request, 'users/leaderdetail.html', {'user': user})


@login_required
def update_leader_detail(request, username):
    user = LeaderUser.objects.get(username__iexact=username)
    form_data = {'username': user.username, 'first_name': user.first_name, 'last_name': user.last_name,
                 'email': user.email,
                 'password': user.password, 'bio': user.bio}
    form = UpdateLeaderForm(request.POST, initial=form_data)
    if request.method == 'POST':
        print(form.errors)
        if form.is_valid():
            user.first_name = request.POST.get('first_name')
            user.last_name = request.POST.get('last_name')
            user.email = request.POST.get('email')
            pw = request.POST['password']
            if (pw is not '' or pw is not None) and len(pw.strip()) >= 8:
                user.set_password(pw)
            user.bio = request.POST.get('bio')
            user.save()
            messages.add_message(request, messages.WARNING, 'User Profile has been altered.\n'
                                                            'Please login again.')
            logout(request)
            update_session_auth_hash(request, request.user)
            return redirect('display_leader', username)

    return render(request, 'users/update_leader_form.html', {'user': user, 'form': form, 'errors': form.errors})



# @author: Abhi Rudra
# utilities for the backend services.
import schedule
from django.core.mail import send_mail, send_mass_mail
from haikunator import haikunator
from twilio.rest import TwilioRestClient
from datetime import datetime, timedelta
import time
import _thread
import os

from log.Log import log
from users.models import MemberUser

sys_email = 'hex.omega@yandex.com'


def get_default_password():
    h = haikunator.Haikunator()
    return h.haikunate()


def mail_kickoff(*args, var=1, **kwargs):
    if var is 1:
        _thread.start_new_thread(send_default_password_threads, (args[0], args[1]))
    elif var is 2:
        _thread.start_new_thread(send_reminder_threads, (args[0],))
    elif var is 3:
        _thread.start_new_thread(send_upload_update, (args[0], args[1]))


def send_upload_update(user, task):
    subject = 'Upload Update - ' + task.title
    body = 'Deliverable for task:[{}] has been uploaded by {}'.format(task.title, user.get_full_name())
    l = [m.email for m in task.members.all()]
    l.append(task.action_list.project.leader.email)
    l.extend([a.email for a in task.action_list.project.admins.all()])
    send_mail(
        subject,
        body,
        sys_email,
        l,
        fail_silently=False
    )


def send_default_password_threads(user, password, **kwargs):
    subject = 'Password - Hex Omega Systems'
    body = 'Password for {}({}) is {}\n\n'.format(user.get_full_name(), user.username, password)
    print(password)
    body += 'Please login and change your password, under Profile->Edit Profile.\n'
    send_mail(
        subject,
        body,
        sys_email,
        [user.email],
        fail_silently=False
    )


def start_schedule_thread():
    print('Yo')
    _thread.start_new_thread(tasks_email_schedule, ())


def tasks_email_schedule():
    from users.models import Project
    print('Haha')
    for project in Project.objects.all():
        print(project.name)
        send_mail(
            'test_scheduler',
            'This is a test for the scheduler. \nPlease ignore.\n' + project.name,
            sys_email,
            ['avsv96@gmail.com'],
            fail_silently=False
        )
        lp = []
        for task in project.actionlist.task_set.all():
            if task.est_end.date() < datetime.now().date():
                for m in task.members.all():
                    log('WARNING', m, '{} is late in submission of deliverable for {}'.format(m.username, task.title))
                if task.to_leader is True:
                    log('WARNING', task.action_list.project.leader,
                        '{} is late in submission of deliverable for {}'.format(
                            task.action_list.project.leader.username, task.title))
            if task.est_end.date() - timedelta(days=1) == datetime.now().date():
                if task.status is not 'Completed' and task.status is not 'Unassigned':
                    l = [member.email for member in task.members.all() if member.email is not '']
                    if task.to_leader:
                        l.append(task.action_list.project.leader.email)
                    sub = task.action_list.project.name + ' : ' + task.title
                    msg = 'This is an automated reminder to submit your deliverable before tomorrow.\n\n'
                    msg += 'Please do not reply to this mail.'
                    t = (sub, msg, sys_email, l)
                    print(t, file=open('mass_mail_log.txt', 'w+'))
                    lp.append(t)

        if len(lp) is not 0:
            mail_kickoff(lp, var=2)
            print(lp, file=open('tasks.txt', 'w+'))


def send_reminder_threads(mails, **kwargs):
    send_mass_mail(
        tuple(mails),
        fail_silently=False
    )


def uploaded_file_handler(f, path):
    with open(os.path.join(path, f.name), 'wb') as destination:
        for chunk in f.chunks():
            destination.write(chunk)


def project_deadline_schedule():
    from users.models import Project
    print('Haha')
    for project in Project.objects.all():
        print(project.name)
        send_mail(
            'test_scheduler',
            'This is a test for the scheduler. \nPlease ignore.\n' + project.name,
            sys_email,
            ['avsv96@gmail.com'],
            fail_silently=False
        )
        lp = []
        if project.end_date == datetime.now() + timedelta(days=2):
            l = [member.email for member in MemberUser.objects.filter(project__name=project.name)]
            l.append(project.leader.email)
            sub = project.name + ' Project Closing Notice'
            msg = 'This is an automated alert. This project will be closed in two days.\n\n'
            msg += 'Please do not reply to this mail.'
            t = (sub, msg, sys_email, l)
            print(t, file=open('mass_mail_log.txt', 'w+'))
            lp.append(t)

        if project.end_date == datetime.now():
            project.status = '1'
            project.save()

        if len(lp) is not 0:
            mail_kickoff(lp, var=2)
            print(lp, file=open('tasks.txt', 'w+'))



from yattag import Doc
from parse import *
import os

from users.models import Project


class parse_log(object):
    def __init__(self, project=None):
        # self.project = project
        # self.logpath = Project.objects.get(name__exact=self.project.name).activitylog.content
        self.logpath = project.activitylog.content
        print(self.logpath)

        if os.path.exists(self.logpath):
            self.logfile = open(self.logpath, 'r')
        else:
            if not os.path.exists('/'.join(self.logpath.split('/')[:-1])):
                print('/'.join(self.logpath.split('/')[:-1]))
                os.makedirs('/'.join(self.logpath.split('/')[:-1]), 0o755)
            self.logfile = open(self.logpath, 'w')
            self.logfile.close()
            self.logfile = open(self.logpath, 'r')
        self.log_data = self.logfile.readlines().reverse()

    def test(self):
        woah = ''

        for line in self.log_data:
            data = parse('[{}] [{}] [{}] [{}] [{}] [{}]', line)

            doc, tag, text = Doc().tagtext()

            if data[0] == 'INFO':
                k = 'info-class'
            elif data[0] == 'WARNING':
                k = 'warning-class'
            else:
                k = 'success-class'
            # if data[0] == 'INFO':
            #     k = 'panel panel-info'
            # elif data[0] == 'WARNING':
            #     k = 'panel panel-warning'
            # else:
            #     k = 'panel panel-warning'

            with tag('div', klass=k):
                # with tag('h4'):
                #     text(data[0])
                with tag('span', klass='username'):
                    text(data[1] + ' ')
                with tag('i'):
                    text(data[4] + ' ')
                with tag('p'):
                    text(data[5])

            woah += doc.getvalue()

        return woah



from .models import User, AdminUser, LeaderUser, MemberUser


class CustomUserAuth(object):
    def authenticate(self, username=None, password=None):
        try:
            user = None
            if AdminUser.objects.filter(username__exact=username).count() == 1:
                # print('ADMIN')
                user = AdminUser.objects.get(username__exact=username)
            elif LeaderUser.objects.filter(username__exact=username).count() == 1:
                # print('LEADER')
                user = LeaderUser.objects.get(username__exact=username)
            elif MemberUser.objects.filter(username__exact=username).count() == 1:
                # print('MEMBER')
                user = MemberUser.objects.get(username__exact=username)
            elif User.objects.filter(username__exact=username).count() == 1:
                # print('USER')
                user = User.objects.get(username__exact=username)

            if user is not None and user.check_password(password):
                return user
            else:
                return False

        except User.DoesNotExist:
            return None

    def get_user(self, user_id):
        try:
            user = None
            if AdminUser.objects.filter(pk=user_id).count() == 1:
                user = AdminUser.objects.get(pk=user_id)
            elif LeaderUser.objects.filter(pk=user_id).count() == 1:
                user = LeaderUser.objects.get(pk=user_id)
            elif MemberUser.objects.filter(pk=user_id).count() == 1:
                user = MemberUser.objects.get(pk=user_id)
            elif User.objects.filter(pk=user_id).count() == 1:
                user = User.objects.get(pk=user_id)

            if user.is_active:
                return user

        except User.DoesNotExist:
            return None



# import threading
#
# import _thread
# import schedule
# import time
# from datetime import datetime, timedelta
#
# from .utils import start_schedule_thread, tasks_email_schedule, project_deadline_schedule
#
#
# def p():
#     print('yo', file=open('yo', 'a'))
#     # tasks_email_schedule()
#     # project_deadline_schedule()
#
#
# print('starting scheduler within users.__init__ ....')
# schedule.every().day.do(p)
#
#
# def run_schedule():
#     while len(schedule.jobs) > 0:
#         schedule.run_pending()
#         time.sleep(72000)
#
#
# _thread.start_new_thread(run_schedule, ())



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-27 09:09
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='role',
            name='title',
            field=models.CharField(max_length=30, unique=True),
        ),
    ]



from django.conf.urls import url
from . import views
from .Claudia import views as cv
from .Xav import views as xv
from log.views import test

urlpatterns = [
    # index page
    url(r'^$', views.index, name="index"),
    # login page
    url(r'^login/$', views.login_auth_2, name='login_page'),
    # log out view
    url(r'^logout/$', views.jump_ship, name='jump_ship'),
    # create project
    url(r'^leader_user/(?P<username>[A-Z0-9][0-9]{7})/create/$', views.create_project, name="create_project"),
    # add task
    url(r'^leader_user/(?P<username>[A-Z0-9][0-9]{7})/add/$', views.create_task, name="add_task"),
    # edit task
    url(r'^leader_user/(?P<username>[A-Z0-9][0-9]{7})/edit/(?P<pk>[0-9]{1,3})/$',
        views.TaskUpdate.as_view(username=''),
        name="leader_update_task"),
    # create member
    url(r'^leader_user/(?P<username>[A-Z0-9][0-9]{7})/add-member/$',
        views.CreateMember.as_view(username=''),
        name="create_member"),

    # members update themselves
    # url(r'^member_user/(?P<username>[A-Z0-9][0-9]{7})/update-member/$', views.update_member, name="update_member"),
    url(r"^member_user/(?P<username>[A-Z0-9][0-9]{7})/update/$", views.update_member, name='update_member'),

    # user delete view
    #url(r'^user/(?P<username>[A-Z0-9][0-9]{7})/delete/(?P<d>[A-Z0-9][0-9]{7})/$', views.delete_admin,
    #    name="delete_admin"),
    # list of users
    url(r'^user/(?P<username>[A-Z0-9][0-9]{7})/list$', views.list_users, name="list_users"),

    # task pages (now dummy pages)
    # the following will show list of open projects for the admin
    url(r'^admin_user/(?P<username>[A-Z0-9][0-9]{7})/$', views.logged_in, name="admin_home"),
    # show, create and edit tasks for specific project
    url(r'^leader_user/(?P<username>[A-Z0-9][0-9]{7})/$', views.leader_home, name="leader_home"),
    # Show member details.
    url(r'^member_user/(?P<username>[A-Z0-9][0-9]{7})/$', views.MemberHome.as_view(username=''), name="member_home"),

    # Show tasks assigned to specific user and able to upload deliverable.
    url(r'^member_user/(?P<username>[A-Z0-9][0-9]{7})/tasks$', views.show_tasks, name="task_list"),
    # user dispatch
    url(r'^user/(?P<username>[A-Z0-9][0-9]{7})/$', views.logged_in, name="user_logged_in"),

    # upload deliverable
    url(r'^member_user/(?P<username>[A-Z0-9][0-9]{7})/tasks/(?P<task>[A-Za-z0-9&\s]+)/$', views.member_upload,
        name='member_upload'),

    # ----- Claudia
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/add_admin/$", cv.create_admin_user, name='add_admin'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/details/$", cv.get_admin_detail, name='display_admin'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/update/$", cv.update_admin_detail, name='update_admin'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/search/$", cv.search_users, name='users_search'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/list/$", cv.get_list_of_users, name='list_of_users'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/update_user/(?P<user>[A-Z0-9][0-9]{7})/$", cv.user_update,
        name='update_user'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/user_detail/(?P<user>[A-Z0-9][0-9]{7})/$", cv.get_user_detail,
        name='user_detail'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/delete/(?P<d>[A-Z0-9][0-9]{7})/$", views.delete_user,
        name='delete_user'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/project/$", cv.display_all_projects, name='all_project'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/open_project/$", cv.display_open_projects, name='open_project'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/(?P<p>[a-zA-Z0-9\s]*)/project_detail/$",
        views.project_information, name='project_detail'),
    url(r"^member_user/(?P<username>[A-Z0-9][0-9]{7})/details/$", cv.get_member_detail, name='display_member'),
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/delete_project/(?P<d>[a-zA-Z0-9\s]*)/$", cv.delete_project,
        name='delete_project'),

    # ----- Xav
    # Creating a leader falls under the admin's role.
    url(r"^admin_user/(?P<username>[A-Z0-9][0-9]{7})/add_leader/$", xv.create_leader_user, name='add_leader'),
    url(r"^leader_user/(?P<username>[A-Z0-9][0-9]{7})/details/$", xv.display_leader_detail, name='display_leader'),
    url(r"^leader_user/(?P<username>[A-Z0-9][0-9]{7})/update/$", xv.update_leader_detail, name='update_leader'),
    # ----- Other
    url(r'^user/list/$', views.get_list_of_users, name='list_of_users'),

    # Log Update

    url(r'^leader_user/(?P<username>[A-Z0-9][0-9]{7})/(?P<project>[A-Za-z0-9\s]*)/log/$', test, name='leader_log'),
    url(r'^admin_user/(?P<username>[A-Z0-9][0-9]{7})/(?P<project>[A-Za-z0-9\s]*)/log/$', test, name='admin_log'),

    url(r'^leader_user/(?P<username>[A-Z0-9][0-9]{7})/list/$', views.get_list_of_members, name='members_list'),

    url(r'^leader_user/(?P<username>[A-Z0-9][0-9]{7})/delete/(?P<d>[A-Z0-9][0-9]{7})/$', views.delete_a_member,
        name='delete_member'),

    url(r'^admin_user/(?P<username>[A-Z0-9][0-9]{7})/(?P<p>[a-zA-Z0-9\s]*)/project_detail/(?P<task_pk>[0-9]+)$',
        views.send_file, name='download_deliverable')

]



from django.db import IntegrityError

from users.views import *

from .user_form import AdminUserForm, AdminUpdateForm, UserUpdateForm

from users.models import User

from users.filters import SearchFilter


@login_required
def create_admin_user(request, username):
    """
    Create an admin user.
    username/add/$
    :param username:
    :param request:
    :return:
    """
    form = AdminUserForm()
    if request.method == 'POST':
        form = AdminUserForm(request.POST)
        if form.is_valid():
            username = request.POST.get('username')
            first_name = request.POST.get('first_name')
            last_name = request.POST.get('last_name')
            email = request.POST.get('email')
            password = get_default_password()
            # bio = request.POST.get('bio')
            user = AdminUser.objects.create(username=username, first_name=first_name, last_name=last_name,
                                            email=email)
            user.set_password(password)
            mail_kickoff(user, password)
            user.save()
            update_session_auth_hash(request, request.user)
            return redirect('display_admin', request.user.username)
    return render(request, 'users/adminuser_form.html', {'form': form})


@login_required
def get_admin_detail(request, username):
    """
    Display the information of an admin user
    :param request:
    :return:
    """
    user = AdminUser.objects.get(username__iexact=username)
    return render(request, 'users/user_information.html', {'adminuser': user})


@login_required
def get_member_detail(request, username):
    """
    Display the information of a member user
    :param request:
    :param username:
    :return:
    """
    user = MemberUser.objects.get(username__iexact=username)
    return render(request, 'users/member_information.html', {'memberuser': user})


# abhi's test decorator : removed from repo
# @viewing_context
@login_required
def update_admin_detail(request, username):
    """
    Update the information of an admin user from "edit profile"
    :param request:
    :param username:
    :return:
    """
    user = AdminUser.objects.get(username__iexact=username)
    form_data = {'first_name': user.first_name, 'last_name': user.last_name,
                 'email': user.email, 'password': " ", 'bio': user.bio}
    form = AdminUpdateForm(request.POST, initial=form_data)
    if request.method == 'POST':
        print(form.errors)
        if form.is_valid():
            user.first_name = request.POST.get('first_name')
            user.last_name = request.POST.get('last_name')
            user.email = request.POST.get('email')
            p = request.POST['password']
            if (p is not '' or p is not None) and len(p.strip()) >= 8:
                user.set_password(p)
            user.bio = request.POST.get('bio')
            user.save()
            update_session_auth_hash(request, request.user)
            return redirect('display_admin', username)

    return render(request, 'users/update_admin_form.html', {'adminuser': user, 'form': form, 'errors': form.errors})


@login_required
def display_all_projects(request, username):
    """
    Display all projects
    :param request:
    :param username:
    :return:
    """
    project_list = Project.objects.all().order_by('pk')
    proj_paginator = Paginator(project_list, 10)
    proj_page = request.GET.get('page')

    try:
        all_project_list = proj_paginator.page(proj_page)
    except PageNotAnInteger:
        all_project_list = proj_paginator.page(1)
    except EmptyPage:
        all_project_list = proj_paginator.page(proj_paginator.num_pages)

    return render(request, 'users/all_project_list.html', {'all_project_list': all_project_list, 'page': proj_page})


@login_required
def display_open_projects(request, username):
    """
    Display a list of open projects
    :param request:
    :param username:
    :return:
    """
    open_proj_list = Project.objects.filter(status='0').order_by('pk')
    open_proj_paginator = Paginator(open_proj_list, 10)
    open_proj_page = request.GET.get('page')

    try:
        open_project_list = open_proj_paginator.page(open_proj_page)
    except PageNotAnInteger:
        open_project_list = open_proj_paginator.page(1)
    except EmptyPage:
        open_project_list = open_proj_paginator.page(open_proj_paginator.num_pages)
    return render(request, 'users/open_project_list.html',
                  {'open_project_list': open_project_list, 'page': open_proj_page})


@login_required
def project_information(request, username, p):
    """
    Display a particular project information
    :param request:
    :param username:
    :param p:
    :return:
    """
    project = Project.objects.get(name__exact=p)
    for p in project.actionlist.task_set.all():
        print(p.deliverable.name.split('/')[-1])
    return render(request, 'users/project_information.html', {'project': project})


@login_required
def user_update(request, username, user):
    """
    Update a user particulars (email and password)
    :param request:
    :param username:
    :param user:
    :return:
    """
    adm = AdminUser.objects.get(username=username)
    person = User.objects.get(username__iexact=user)
    form_data = {'email': person.email, 'password': " "}
    form = UserUpdateForm(request.POST, initial=form_data)
    if request.method == 'POST':
        if form.is_valid() and user != '10101010':
            person.email = request.POST.get('email')
            pw = request.POST['password']
            if (pw is not '' or pw is not None) and len(pw.strip()) >= 8:
                person.set_password(pw)
            person.save()
            update_session_auth_hash(request, request.user)
            return redirect('list_of_users', username)
    return render(request, 'users/user_update_form.html', {'form': form, 'errors': form.errors, 'user': person})


@login_required
def get_list_of_users(request, username):
    """
    Display a list of all users (admins, leaders and members)
    :param request:
    :param username:
    :return:
    """
    person = User.objects.get(username__iexact=username)
    list_of_users = User.objects.exclude(username__iexact='AnonymousUser')
    paginator = Paginator(list_of_users, 10)
    page = request.GET.get('page')
    try:
        all_users_list = paginator.page(page)
    except PageNotAnInteger:
        all_users_list = paginator.page(1)
    except EmptyPage:
        all_users_list = paginator.page(paginator.num_pages)

    context = {'user': person, 'all_users_list': all_users_list, 'page': page}
    return render(request, 'users/list_of_users.html', context)


@login_required
def get_user_detail(request, username, user):
    """
    Display the user detail from the list of all users
    :param request:
    :param username:
    :param user:
    :return:
    """
    person = User.objects.get(username__iexact=user)
    return render(request, 'users/user_information.html', {'user': person})


@login_required
def search_users(request, username):
    """
    Search for all users
    :param request:
    :param username:
    :return:
    """
    if 'username' in request.GET or 'first_name' in request.GET or 'last_name' in request.GET:
        queryset = User.objects.exclude(username__iexact='AnonymousUser')
        result_list = SearchFilter(request.GET, queryset=queryset)
        return render(request, 'users/search.html', {'result_list': result_list})
    return render(request, 'users/search.html')


def delete_project(request, username, d):
    """
    Delete a project from the system
    :param request:
    :param username:
    :param d:
    :return:
    """
    if Project.objects.get(name__iexact=d):
        project = Project.objects.get(name__iexact=d)
        try:
            project.delete()
        except IntegrityError as ie:
            messages.add_message(request, messages.WARNING,
                                 'A project consists of members.\n'
                                 'Please delete all members prior to deleting a project.')
    return redirect('all_project', username)



from celery import Celery
from celery.schedules import crontab

from .utils import tasks_email_schedule, project_deadline_schedule

app = Celery()


@app.on_after_configure.connect
def setup_periodic_tasks(sender, **kwargs):
    # Calls test('hello') every 10 seconds.
    sender.add_periodic_task(5.0, test.s('hello'), name='add every 10')

    # Calls test('world') every 30 seconds
    sender.add_periodic_task(30.0, test.s('world'), expires=10)

    # sender.add_periodic_task(
    #     crontab(hour=13, minute=57),
    #     test.s('I want to hold your hand'),
    # )

    # sender.add_periodic_task(
    #     crontab(hour=7, minute=30),
    #     test.s(),
    # )


@app.task
def test(arg):
    f = open('help.txt', 'a')
    print(arg, file=f)
    f.close()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-27 09:05
from __future__ import unicode_literals

from django.conf import settings
import django.contrib.auth.models
import django.contrib.auth.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import guardian.mixins
import users.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=30, verbose_name='last name')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('bio', models.TextField(blank=True, max_length=500)),
                ('phone', models.TextField(blank=True, max_length=15)),
                ('email', models.EmailField(blank=True, max_length=254, unique=True)),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'abstract': False,
            },
            bases=(models.Model, guardian.mixins.GuardianUserMixin),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='ActionList',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('description', models.TextField(blank=True, max_length=500)),
            ],
            options={
                'db_table': 'ActionList',
            },
        ),
        migrations.CreateModel(
            name='ActivityLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=30)),
                ('content', models.TextField(blank=True, max_length=100, null=True)),
                ('timestamp', models.DateTimeField(auto_now=True)),
            ],
            options={
                'db_table': 'ActivityLog',
            },
        ),
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True)),
                ('status', models.CharField(choices=[('Open', 'Open'), ('Closed', 'Closed')], default=0, max_length=7)),
                ('start_date', models.DateField(blank=True)),
                ('end_date', models.DateField(blank=True)),
                ('description', models.TextField(blank=True, max_length=500)),
            ],
            options={
                'db_table': 'Project',
            },
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=30)),
            ],
            options={
                'db_table': 'Role',
            },
        ),
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('Draft', 'Draft'), ('Final', 'Final'), ('Unassigned', 'Unassigned'), ('Assigned', 'Assigned'), ('Completed', 'Completed'), ('Late', 'Late'), ('Not Submitted', 'Not Submitted')], default=2, max_length=14)),
                ('start_date', models.DateTimeField(auto_now=True)),
                ('est_end', models.DateTimeField(blank=True, null=True)),
                ('actual_end', models.DateTimeField(blank=True, null=True)),
                ('description', models.TextField(blank=True, max_length=500, null=True)),
                ('link', models.URLField(blank=True, null=True)),
                ('title', models.CharField(blank=True, max_length=50, unique=True)),
                ('deliverable', models.FileField(blank=True, null=True, upload_to=users.models.get_path)),
                ('to_leader', models.NullBooleanField(verbose_name='Is the leader working on this Task?')),
                ('action_list', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='users.ActionList')),
            ],
            options={
                'db_table': 'Task',
            },
        ),
        migrations.CreateModel(
            name='AdminUser',
            fields=[
                ('user_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'AdminUser',
                'permissions': (('can_create_admin', 'Can create Admin'),),
            },
            bases=('users.user', models.Model),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='LeaderUser',
            fields=[
                ('user_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'Leader',
                'permissions': (('can_create_leader', 'Can create Team Leader'),),
            },
            bases=('users.user', models.Model),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='MemberUser',
            fields=[
                ('user_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'Member',
                'permissions': (('can_create_member', 'Can create Team Member'),),
            },
            bases=('users.user', models.Model),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.AddField(
            model_name='activitylog',
            name='project',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='users.Project'),
        ),
        migrations.AddField(
            model_name='actionlist',
            name='project',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='users.Project'),
        ),
        migrations.AddField(
            model_name='user',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups'),
        ),
        migrations.AddField(
            model_name='user',
            name='user_permissions',
            field=models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions'),
        ),
        migrations.AddField(
            model_name='task',
            name='members',
            field=models.ManyToManyField(to='users.MemberUser'),
        ),
        migrations.AddField(
            model_name='project',
            name='admins',
            field=models.ManyToManyField(to='users.AdminUser'),
        ),
        migrations.AddField(
            model_name='project',
            name='leader',
            field=models.OneToOneField(on_delete=django.db.models.deletion.DO_NOTHING, to='users.LeaderUser'),
        ),
        migrations.AddField(
            model_name='memberuser',
            name='project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='users.Project'),
        ),
        migrations.AddField(
            model_name='memberuser',
            name='role',
            field=models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='users.Role'),
        ),
    ]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='hexomega',
    version='0.2',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',  # example license
    description='A package to streamline creation of projects and tasks for Murdoch University.',
    long_description=README,
    url='https://www.github.com/defstryker/Hex-Omega',
    author='Abhi Rudra',
    author_email='defstryker@hotmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.10', 
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        # Replace these appropriately if you are stuck on Python 2.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
    	'schedule',
		'django',
		'mysqlclient',
		'whitenoise',
		'django-guardian',
		'django-annoying',
		'parse',
		'yattag',
		'haikunator',
		'django-bootstrap3',
		'django-bootstrap-datepicker',
		'django-filter',
		'dj_database_url',
		'psycopg2',
    ],
)


from django import forms
from users.models import AdminUser


class AdminUserForm(forms.Form):
    username = forms.CharField(label='Username', max_length=10)
    first_name = forms.CharField(label='First Name', max_length=30)
    last_name = forms.CharField(label='Last Name', max_length=30)
    email = forms.EmailField(label='Email', widget=forms.EmailInput)
    password = forms.CharField(label='Password', widget=forms.PasswordInput, min_length=6, required=False)
    bio = forms.CharField(label='Bio', widget=forms.Textarea)

    class Meta:
        model = AdminUser


# It was I, It was needed. Change is inevitable.
class AdminUpdateForm(forms.Form):
    # username = forms.CharField(label='Username', max_length=10)
    first_name = forms.CharField(label='First Name', max_length=30)
    last_name = forms.CharField(label='Last Name', max_length=30)
    email = forms.EmailField(label='Email', widget=forms.EmailInput)
    password = forms.CharField(label='Password', widget=forms.PasswordInput, min_length=6, required=False)
    bio = forms.CharField(label='Bio', widget=forms.Textarea)

    class Meta:
        model = AdminUser


class PasswordForm(forms.Form):
    password = forms.CharField(label='Password', widget=forms.PasswordInput, min_length=6, required=False)



from django.http import HttpResponseForbidden


def url_context(function):
    def wrapper(request, *args, **kwargs):
        user = request.user
        if request.user.username not in request.get_full_path():
            return HttpResponseForbidden()
        else:
            return function(request, *args, **kwargs)
    return wrapper



from django import forms
from users.models import LeaderUser


class LeaderForm(forms.Form):
    username = forms.CharField(label='Username', max_length=8, min_length=8)
    first_name = forms.CharField(label='First Name', max_length=30)
    last_name = forms.CharField(label='Last Name', max_length=30)
    email = forms.EmailField(label='Email')

    def clean_email(self):
        data = self.cleaned_data['email']
        if LeaderUser.objects.filter(email=data).exists():
            raise forms.ValidationError("This email already used")
        return data

    class Meta:
        model = LeaderUser


class UpdateLeaderForm(forms.Form):
    first_name = forms.CharField(label='First Name', max_length=30)
    last_name = forms.CharField(label='Last Name', max_length=30)
    email = forms.EmailField(label='Email', widget=forms.EmailInput)
    password = forms.CharField(label='Password', widget=forms.PasswordInput, min_length=6, required=False)
    bio = forms.CharField(label='Bio', widget=forms.Textarea)

    def clean_email(self):
        data = self.cleaned_data['email']
        if self.has_changed() is False:
            if LeaderUser.objects.filter(email=data).exists():
                raise forms.ValidationError("This email already used")
            return data
        else:
            return data

    class Meta:
        model = LeaderUser



from django import forms
from django.contrib.admin.widgets import AdminDateWidget, FilteredSelectMultiple, AdminSplitDateTime
from django.forms import ModelForm

from users.models import MemberUser, Task


class CreateTaskForm(ModelForm):
    def __init__(self, *args, **kwargs):
        super(ModelForm, self).__init__(*args, **kwargs)
        ms = MemberUser.objects.filter(project__name=self.data.get('pn'))
        c = [(m.id, m.get_full_name()) for m in ms]
        self.fields['members'].choices = c
        self.fields['members'].required = False
        self.fields['title'].required = True

    class Meta:
        model = Task
        fields = ['title', 'status', 'est_end', 'to_leader', 'members']

        widgets = {
            # 'est_end': forms.DateTimeInput(attrs={'class': 'datetime-input'}),
            'est_end': AdminDateWidget(),
            'members': forms.CheckboxSelectMultiple()
        }
        help_texts = {
            'est_end': "Please use this format: 'YYYY-MM-DD'",
            'users': 'Hold down Control/Cmd to select more than member for the task.'
        }

    def clean(self):
        print('United Abominations')
        cleaned_data = super(ModelForm, self).clean()
        return cleaned_data


class LeaderUpdateTaskForm(ModelForm):
    class Meta:
        model = Task
        fields = ['title', 'status', 'est_end', 'description', 'deliverable', 'to_leader', 'members']

        widgets = {
            'est_end': AdminDateWidget(),
            'members': forms.CheckboxSelectMultiple()
        }

    def __init__(self, *args, **kwargs):
        pn = kwargs.pop('pn')
        up_flag = kwargs.pop('up_flag')
        up_name = kwargs.pop('up_name')
        qs = MemberUser.objects.filter(project__name=pn)
        c = [(m.id, m.get_full_name()) for m in qs]
        super(LeaderUpdateTaskForm, self).__init__(*args, **kwargs)
        self.fields['members'].choices = c
        self.fields['members'].required = False
        self.fields['title'].required = True
        if up_flag:
            self.fields.pop('deliverable')
            self.fields['Deliverable'] = forms.CharField(max_length=50, required=False)
            self.fields['Deliverable'].initial = "Deliverable [{}] already uploaded.".format(up_name)
            self.fields['Deliverable'].widget.attrs['readonly'] = True

            self.fields['title'].widget.attrs['readonly'] = True

            self.initial['status'] = 'Completed'
            self.fields['status'] = forms.CharField(max_length=50, required=False)
            self.fields['status'].widget.attrs['readonly'] = True

            self.fields['est_end'] = forms.CharField(max_length=50, required=False)
            self.fields['est_end'].widget.attrs['readonly'] = True

            self.fields['description'] = forms.CharField(max_length=50, required=False)
            self.fields['description'].widget.attrs['readonly'] = True

            self.fields['to_leader'] = forms.CharField(max_length=50, required=False)
            self.fields['to_leader'].widget.attrs['readonly'] = True

            self.fields['members'] = forms.CharField(max_length=50, required=False)
            names = ''
            for m in self.initial['members']:
                names += m.get_full_name()
                names += ', '
            self.initial['members'] = names[:-2]
            self.fields['members'].widget.attrs['readonly'] = True



from django.test import TestCase
from parse import *

from datetime import datetime, timedelta

from log.Log import log
from users.models import *


# Create your tests here.
class LogTest(TestCase):
    def setUp(self):
        # Create role.
        r = Role()
        r.title = 'Security Coordinator'
        r.save()

        # Create admin
        adm = AdminUser(username='G2503472', first_name='admin', last_name='man')
        adm.email = 'admin_man@example.com'
        adm.set_password('qwerty123')
        adm.save()

        # Create leader
        l = LeaderUser(username='69497604', first_name='leader', last_name='man')
        l.email = 'leader_man@example.com'
        l.set_password('qwerty123')
        l.save()

        # Create project
        p = Project(name='Test')
        p.start_date = datetime.now()
        p.end_date = datetime.now() + timedelta(days=30)
        p.leader_id = l.id
        p.save()
        p.admins.add(adm)
        p.save()

        # Create member
        m = MemberUser(username='56475644', first_name='Heracles', last_name='Alcmene')
        m.set_password('qwerty123')
        m.email = 'heracles@example.com'
        m.role_id = r.id
        m.project_id = p.id
        m.save()

        n = MemberUser(username='40475328', first_name='Perseus', last_name='Son of Danae')
        n.set_password('qwerty123')
        n.email = 'perseus@example.com'
        n.role_id = r.id
        n.project_id = p.id
        n.save()

        o = MemberUser(username='81343691', first_name='Apollo', last_name='Son of Leto')
        o.set_password('qwerty123')
        o.email = 'apollo@example.com'
        o.role_id = r.id
        o.project_id = p.id
        o.save()

    def Test_log_contains_info_member(self):
        m = MemberUser.objects.get(first_name__exact='Heracles')
        p = m.project
        log('INFO', m, 'test content')
        f = open(p.activitylog.content, 'r')
        logfile = f.readlines()[0]
        data = parse('[{}] [{}] [{}] [{}] [{}] [{}]', logfile)
        TestCase.assertEquals(self, data[0], 'INFO')
        TestCase.assertEquals(self, data[1], m.get_full_name())
        TestCase.assertEquals(self, data[2], 'MEMBER')
        TestCase.assertEquals(self, data[3], m.project.name)
        TestCase.assertEquals(self, data[5], 'test content')
        f.flush()
        f.close()

    def Test_log_contains_warning_leader(self):
        l = LeaderUser.objects.get(first_name__exact='leader')
        p = l.project
        log('WARNING', l, 'leader log record')
        f = open(p.activitylog.content, 'r')
        logfile = f.readlines()[1]
        data = parse('[{}] [{}] [{}] [{}] [{}] [{}]', logfile)
        TestCase.assertEquals(self, data[0], 'WARNING')
        TestCase.assertEquals(self, data[1], l.get_full_name())
        TestCase.assertEquals(self, data[2], 'LEADER')
        TestCase.assertEquals(self, data[3], l.project.name)
        TestCase.assertEquals(self, data[5], 'leader log record')
        f.flush()
        f.close()

    def test_log_contains_all_records(self):
        """
        We have added two records, one each for leader and admin.
        This functions asserts if the log file contains two records.
        To make it more comprehensive, after the initial assertion,
        another record is add to the log and the number of records
        are tested once again.

        It calls the previous two methods so the logs are within the same
        execution context.
        :return:
        """
        self.Test_log_contains_info_member()
        self.Test_log_contains_warning_leader()

        p = Project.objects.get(name__exact='Hell')

        f = open(p.activitylog.content, 'r')
        logfile = f.readlines()
        number_of_lines = len(logfile)
        TestCase.assertEquals(self, number_of_lines, 2)
        f.close()
        os.remove(p.activitylog.content)
        s = '/'.join(p.activitylog.content.split('/')[:-1])
        os.rmdir(s)



from django import forms
from users.models import AdminUser, User


class AdminUserForm(forms.Form):
    username = forms.CharField(label='Username', max_length=8, min_length=8)
    first_name = forms.CharField(label='First Name', max_length=30)
    last_name = forms.CharField(label='Last Name', max_length=30)
    email = forms.EmailField(label='Email', widget=forms.EmailInput)
    bio = forms.CharField(label='Bio', widget=forms.Textarea, required=False)

    def clean_username(self):
        username = self.cleaned_data.get('username')
        if AdminUser.objects.filter(username=username).exists():
            raise forms.ValidationError("Username exists in the system.")
        return username

    def clean_email(self):
        email = self.cleaned_data.get('email')
        if AdminUser.objects.filter(email=email).exists():
            raise forms.ValidationError("Email exists in the system.")
        return email

    class Meta:
        model = AdminUser


class AdminUpdateForm(forms.Form):
    first_name = forms.CharField(label='First Name', max_length=30)
    last_name = forms.CharField(label='Last Name', max_length=30)
    email = forms.EmailField(label='Email', widget=forms.EmailInput)
    password = forms.CharField(label='Password', widget=forms.PasswordInput, min_length=6, required=False)
    bio = forms.CharField(label='Bio', widget=forms.Textarea, required=False)

    class Meta:
        model = AdminUser

    def clean_email(self):
        if 'email' in self.changed_data:
            if AdminUser.objects.filter(email=self.cleaned_data.get('email')).exists():
                raise forms.ValidationError("Email exists in the system.")
            print('changed!')
        else:
            print('no!')
        return self.cleaned_data.get('email')


class UserUpdateForm(forms.Form):
    email = forms.EmailField(label='Email', widget=forms.EmailInput)
    password = forms.CharField(label='Password', widget=forms.PasswordInput, min_length=6, required=False)

    def clean_email(self):
        if 'email' in self.changed_data:
            if User.objects.filter(email=self.cleaned_data.get('email')).exists():
                raise forms.ValidationError("Email exists in the system.")
            print('changed!')
        else:
            print('no!')
        return self.cleaned_data.get('email')



from django.contrib import admin

from .models import *

# Register your models here.
admin.site.register(AdminUser)
admin.site.register(LeaderUser)
admin.site.register(Role)
admin.site.register(ActionList)
admin.site.register(Task)
admin.site.register(ActivityLog)
admin.site.register(Project)
admin.site.register(MemberUser)
